@@ -4,16 +4,21 @@
 //! in which this application entity listens to incoming association requests.
 //! See [`ServerAssociationOptions`](self::ServerAssociationOptions)
 //! for details and examples on how to create an association.
-use std::{borrow::Cow, net::TcpStream};
+use std::{borrow::Cow, io::Write, net::TcpStream};
 
 use dicom_encoding::transfer_syntax::TransferSyntaxIndex;
 use dicom_transfer_syntax_registry::TransferSyntaxRegistry;
 use snafu::{ensure, ResultExt, Snafu};
 
-use crate::pdu::{
-    reader::read_pdu, writer::write_pdu, AbortRQServiceProviderReason, AbortRQSource,
-    AssociationRJResult, AssociationRJServiceUserReason, AssociationRJSource, Pdu,
-    PresentationContextResult, PresentationContextResultReason,
+use crate::{
+    pdu::{
+        reader::{read_pdu, DEFAULT_MAX_PDU, MAXIMUM_PDU_SIZE},
+        writer::write_pdu,
+        AbortRQServiceProviderReason, AbortRQSource, AssociationRJResult,
+        AssociationRJServiceUserReason, AssociationRJSource, Pdu, PresentationContextResult,
+        PresentationContextResultReason, UserVariableItem,
+    },
+    IMPLEMENTATION_CLASS_UID, IMPLEMENTATION_VERSION_NAME,
 };
 
 use super::pdata::PDataWriter;
@@ -30,9 +35,12 @@ pub enum Error {
     /// failed to send association response
     SendResponse { source: crate::pdu::writer::Error },
 
-    /// failed to send PDU
+    /// failed to prepare PDU
     Send { source: crate::pdu::writer::Error },
 
+    /// failed to send PDU over the wire
+    WireSend { source: std::io::Error },
+
     /// failed to receive PDU
     Receive { source: crate::pdu::reader::Error },
 
@@ -55,6 +63,13 @@ pub enum Error {
 
     /// association aborted
     Aborted,
+
+    #[snafu(display(
+        "PDU is too large ({} bytes) to be sent to the remote application entity",
+        length
+    ))]
+    #[non_exhaustive]
+    SendTooLongPdu { length: usize },
 }
 
 pub type Result<T, E = Error> = std::result::Result<T, E>;
@@ -288,12 +303,15 @@ where
 
     /// Negotiate an association with the given TCP stream.
     pub fn establish(&self, mut socket: TcpStream) -> Result<ServerAssociation> {
-        ensure!(!self.abstract_syntax_uids.is_empty(), MissingAbstractSyntax);
+        ensure!(
+            !self.abstract_syntax_uids.is_empty(),
+            MissingAbstractSyntaxSnafu
+        );
 
         let max_pdu_length = self.max_pdu_length;
 
-        let pdu = read_pdu(&mut socket, max_pdu_length).context(ReceiveRequest)?;
-
+        let pdu = read_pdu(&mut socket, max_pdu_length, true).context(ReceiveRequestSnafu)?;
+        let mut buffer: Vec<u8> = Vec::with_capacity(max_pdu_length as usize);
         match pdu {
             Pdu::AssociationRQ {
                 protocol_version,
@@ -301,11 +319,11 @@ where
                 called_ae_title,
                 application_context_name,
                 presentation_contexts,
-                user_variables: _,
+                user_variables,
             } => {
                 if protocol_version != self.protocol_version {
                     write_pdu(
-                        &mut socket,
+                        &mut buffer,
                         &Pdu::AssociationRJ {
                             result: AssociationRJResult::Permanent,
                             source: AssociationRJSource::ServiceUser(
@@ -313,13 +331,14 @@ where
                             ),
                         },
                     )
-                    .context(SendResponse)?;
-                    return Rejected.fail();
+                    .context(SendResponseSnafu)?;
+                    socket.write_all(&buffer).context(WireSendSnafu)?;
+                    return RejectedSnafu.fail();
                 }
 
                 if application_context_name != self.application_context_name {
                     write_pdu(
-                        &mut socket,
+                        &mut buffer,
                         &Pdu::AssociationRJ {
                             result: AssociationRJResult::Permanent,
                             source: AssociationRJSource::ServiceUser(
@@ -327,8 +346,9 @@ where
                             ),
                         },
                     )
-                    .context(SendResponse)?;
-                    return Rejected.fail();
+                    .context(SendResponseSnafu)?;
+                    socket.write_all(&buffer).context(WireSendSnafu)?;
+                    return RejectedSnafu.fail();
                 }
 
                 self.ae_access_control
@@ -336,16 +356,33 @@ where
                     .map(Ok)
                     .unwrap_or_else(|reason| {
                         write_pdu(
-                            &mut socket,
+                            &mut buffer,
                             &Pdu::AssociationRJ {
                                 result: AssociationRJResult::Permanent,
                                 source: AssociationRJSource::ServiceUser(reason),
                             },
                         )
-                        .context(SendResponse)?;
-                        Rejected.fail()
+                        .context(SendResponseSnafu)?;
+                        socket.write_all(&buffer).context(WireSendSnafu)?;
+                        RejectedSnafu.fail()
                     })?;
 
+                // fetch requested maximum PDU length
+                let requestor_max_pdu_length = user_variables
+                    .iter()
+                    .find_map(|item| match item {
+                        UserVariableItem::MaxLength(len) => Some(*len),
+                        _ => None,
+                    })
+                    .unwrap_or(DEFAULT_MAX_PDU);
+
+                // treat 0 as the maximum size admitted by the standard
+                let requestor_max_pdu_length = if requestor_max_pdu_length == 0 {
+                    MAXIMUM_PDU_SIZE
+                } else {
+                    requestor_max_pdu_length
+                };
+
                 let presentation_contexts: Vec<_> = presentation_contexts
                     .into_iter()
                     .map(|pc| {
@@ -377,32 +414,46 @@ where
                     .collect();
 
                 write_pdu(
-                    &mut socket,
+                    &mut buffer,
                     &Pdu::AssociationAC {
                         protocol_version: self.protocol_version,
                         application_context_name,
                         presentation_contexts: presentation_contexts.clone(),
-                        user_variables: vec![],
+                        calling_ae_title,
+                        called_ae_title,
+                        user_variables: vec![
+                            UserVariableItem::MaxLength(max_pdu_length),
+                            UserVariableItem::ImplementationClassUID(
+                                IMPLEMENTATION_CLASS_UID.to_string(),
+                            ),
+                            UserVariableItem::ImplementationVersionName(
+                                IMPLEMENTATION_VERSION_NAME.to_string(),
+                            ),
+                        ],
                     },
                 )
-                .context(SendResponse)?;
+                .context(SendResponseSnafu)?;
+                socket.write_all(&buffer).context(WireSendSnafu)?;
 
                 Ok(ServerAssociation {
                     presentation_contexts,
-                    max_pdu_length,
+                    requestor_max_pdu_length,
+                    acceptor_max_pdu_length: max_pdu_length,
                     socket,
+                    buffer,
                 })
             }
             Pdu::ReleaseRQ => {
-                write_pdu(&mut socket, &Pdu::ReleaseRP).context(SendResponse)?;
-                Aborted.fail()
+                write_pdu(&mut buffer, &Pdu::ReleaseRP).context(SendResponseSnafu)?;
+                socket.write_all(&buffer).context(WireSendSnafu)?;
+                AbortedSnafu.fail()
             }
             pdu @ Pdu::AssociationAC { .. }
             | pdu @ Pdu::AssociationRJ { .. }
             | pdu @ Pdu::PData { .. }
             | pdu @ Pdu::ReleaseRP
-            | pdu @ Pdu::AbortRQ { .. } => UnexpectedRequest { pdu }.fail(),
-            pdu @ Pdu::Unknown { .. } => UnknownRequest { pdu }.fail(),
+            | pdu @ Pdu::AbortRQ { .. } => UnexpectedRequestSnafu { pdu }.fail(),
+            pdu @ Pdu::Unknown { .. } => UnknownRequestSnafu { pdu }.fail(),
         }
     }
 }
@@ -422,10 +473,14 @@ where
 pub struct ServerAssociation {
     /// The accorded presentation contexts
     presentation_contexts: Vec<PresentationContextResult>,
-    /// The maximum PDU length
-    max_pdu_length: u32,
+    /// The maximum PDU length that the remote application entity accepts
+    requestor_max_pdu_length: u32,
+    /// The maximum PDU length that this application entity is expecting to receive
+    acceptor_max_pdu_length: u32,
     /// The TCP stream to the other DICOM node
     socket: TcpStream,
+    /// write buffer to send fully assembled PDUs on wire
+    buffer: Vec<u8>,
 }
 
 impl ServerAssociation {
@@ -436,28 +491,32 @@ impl ServerAssociation {
 
     /// Send a PDU message to the other intervenient.
     pub fn send(&mut self, msg: &Pdu) -> Result<()> {
-        write_pdu(&mut self.socket, &msg).context(Send)
+        self.buffer.clear();
+        write_pdu(&mut self.buffer, msg).context(SendSnafu)?;
+        if self.buffer.len() > self.requestor_max_pdu_length as usize {
+            return SendTooLongPduSnafu {
+                length: self.buffer.len(),
+            }
+            .fail();
+        }
+        self.socket.write_all(&self.buffer).context(WireSendSnafu)
     }
 
     /// Read a PDU message from the other intervenient.
     pub fn receive(&mut self) -> Result<Pdu> {
-        read_pdu(&mut self.socket, self.max_pdu_length).context(Receive)
+        read_pdu(&mut self.socket, self.acceptor_max_pdu_length, true).context(ReceiveSnafu)
     }
 
     /// Send a provider initiated abort message
     /// and shut down the TCP connection,
     /// terminating the association.
     pub fn abort(mut self) -> Result<()> {
-        let out = write_pdu(
-            &mut self.socket,
-            &Pdu::AbortRQ {
-                source: AbortRQSource::ServiceProvider(
-                    AbortRQServiceProviderReason::ReasonNotSpecifiedUnrecognizedPdu,
-                ),
-            },
-        )
-        .context(Send);
-
+        let pdu = Pdu::AbortRQ {
+            source: AbortRQSource::ServiceProvider(
+                AbortRQServiceProviderReason::ReasonNotSpecifiedUnrecognizedPdu,
+            ),
+        };
+        let out = self.send(&pdu);
         let _ = self.socket.shutdown(std::net::Shutdown::Both);
         out
     }
@@ -471,7 +530,7 @@ impl ServerAssociation {
         PDataWriter::new(
             &mut self.socket,
             presentation_context_id,
-            self.max_pdu_length,
+            self.requestor_max_pdu_length,
         )
     }
 
