@@ -1,14 +1,27 @@
 /// PDU reader module
 use crate::pdu::*;
 use byteordered::byteorder::{BigEndian, ReadBytesExt};
-use dicom_encoding::text::{SpecificCharacterSet, TextCodec};
+use dicom_encoding::text::{DefaultCharacterSetCodec, TextCodec};
 use snafu::{ensure, Backtrace, OptionExt, ResultExt, Snafu};
 use std::io::{Cursor, ErrorKind, Read, Seek, SeekFrom};
 
+/// The default maximum PDU size
 pub const DEFAULT_MAX_PDU: u32 = 16_384;
+
+/// The minimum PDU size,
+/// as specified by the standard
 pub const MINIMUM_PDU_SIZE: u32 = 4_096;
+
+/// The maximum PDU size,
+/// as specified by the standard
 pub const MAXIMUM_PDU_SIZE: u32 = 131_072;
 
+/// The length of the PDU header in bytes,
+/// comprising the PDU type (1 byte),
+/// reserved byte (1 byte),
+/// and PDU length (4 bytes).
+pub const PDU_HEADER_SIZE: u32 = 6;
+
 #[derive(Debug, Snafu)]
 #[non_exhaustive]
 pub enum Error {
@@ -90,13 +103,13 @@ pub enum Error {
 
 pub type Result<T> = std::result::Result<T, Error>;
 
-pub fn read_pdu<R>(reader: &mut R, max_pdu_length: u32) -> Result<Pdu>
+pub fn read_pdu<R>(reader: &mut R, max_pdu_length: u32, strict: bool) -> Result<Pdu>
 where
     R: Read,
 {
     ensure!(
         (MINIMUM_PDU_SIZE..=MAXIMUM_PDU_SIZE).contains(&max_pdu_length),
-        InvalidMaxPdu { max_pdu_length }
+        InvalidMaxPduSnafu { max_pdu_length }
     );
 
     // If we can't read 2 bytes here, that means that there is no PDU
@@ -106,28 +119,41 @@ where
     // UnexpectedEof if we get after we have already began reading a PDU message.
     let mut bytes = [0; 2];
     if let Err(e) = reader.read_exact(&mut bytes) {
-        ensure!(e.kind() != ErrorKind::UnexpectedEof, NoPduAvailable);
-        return Err(e).context(ReadPduField { field: "type" });
+        ensure!(e.kind() != ErrorKind::UnexpectedEof, NoPduAvailableSnafu);
+        return Err(e).context(ReadPduFieldSnafu { field: "type" });
     }
 
     let pdu_type = bytes[0];
     let pdu_length = reader
         .read_u32::<BigEndian>()
-        .context(ReadPduField { field: "length" })?;
+        .context(ReadPduFieldSnafu { field: "length" })?;
 
-    ensure!(
-        pdu_length <= max_pdu_length,
-        PduTooLarge {
-            pdu_length,
-            max_pdu_length
-        }
-    );
+    // Check max_pdu_length
+    if strict {
+        ensure!(
+            pdu_length <= max_pdu_length,
+            PduTooLargeSnafu {
+                pdu_length,
+                max_pdu_length
+            }
+        );
+    } else if pdu_length > max_pdu_length {
+        ensure!(
+            pdu_length <= MAXIMUM_PDU_SIZE,
+            PduTooLargeSnafu {
+                pdu_length,
+                max_pdu_length: MAXIMUM_PDU_SIZE
+            }
+        );
+        tracing::warn!(
+            "Incoming pdu was too large: length {}, maximum is {}",
+            pdu_length, max_pdu_length
+        );
+    }
 
-    let bytes = read_n(reader, pdu_length as usize).context(ReadPdu)?;
+    let bytes = read_n(reader, pdu_length as usize).context(ReadPduSnafu)?;
     let mut cursor = Cursor::new(bytes);
-    let codec = SpecificCharacterSet::Default
-        .codec()
-        .expect("Support for the default character set is mandatory");
+    let codec = DefaultCharacterSetCodec;
 
     match pdu_type {
         0x01 => {
@@ -142,7 +168,7 @@ where
             // Version 1 and shall be identified with bit 0 set. A receiver of this PDU
             // implementing only this version of the DICOM UL protocol shall only test that bit
             // 0 is set.
-            let protocol_version = cursor.read_u16::<BigEndian>().context(ReadPduField {
+            let protocol_version = cursor.read_u16::<BigEndian>().context(ReadPduFieldSnafu {
                 field: "Protocol-version",
             })?;
 
@@ -150,7 +176,7 @@ where
             // tested to this value when received.
             cursor
                 .read_u16::<BigEndian>()
-                .context(ReadReserved { bytes: 2_u32 })?;
+                .context(ReadReservedSnafu { bytes: 2_u32 })?;
 
             // 11-26 - Called-AE-title - Destination DICOM Application Name. It shall be encoded
             // as 16 characters as defined by the ISO 646:1990-Basic G0 Set with leading and
@@ -158,12 +184,14 @@ where
             // meaning "no Application Name specified" shall not be used. For a complete
             // description of the use of this field, see Section 7.1.1.4.
             let mut ae_bytes = [0; 16];
-            cursor.read_exact(&mut ae_bytes).context(ReadPduField {
-                field: "Called-AE-title",
-            })?;
+            cursor
+                .read_exact(&mut ae_bytes)
+                .context(ReadPduFieldSnafu {
+                    field: "Called-AE-title",
+                })?;
             let called_ae_title = codec
                 .decode(&ae_bytes)
-                .context(DecodeText {
+                .context(DecodeTextSnafu {
                     field: "Called-AE-title",
                 })?
                 .trim()
@@ -175,12 +203,14 @@ where
             // meaning "no Application Name specified" shall not be used. For a complete
             // description of the use of this field, see Section 7.1.1.3.
             let mut ae_bytes = [0; 16];
-            cursor.read_exact(&mut ae_bytes).context(ReadPduField {
-                field: "Calling-AE-title",
-            })?;
+            cursor
+                .read_exact(&mut ae_bytes)
+                .context(ReadPduFieldSnafu {
+                    field: "Calling-AE-title",
+                })?;
             let calling_ae_title = codec
                 .decode(&ae_bytes)
-                .context(DecodeText {
+                .context(DecodeTextSnafu {
                     field: "Calling-AE-title",
                 })?
                 .trim()
@@ -190,7 +220,7 @@ where
             // bytes but not tested to this value when received
             cursor
                 .seek(SeekFrom::Current(32))
-                .context(ReadReserved { bytes: 32_u32 })?;
+                .context(ReadReservedSnafu { bytes: 32_u32 })?;
 
             // 75-xxx - Variable items - This variable field shall contain the following items:
             // one Application Context Item, one or more Presentation Context Items and one User
@@ -208,7 +238,7 @@ where
                         user_variables = val;
                     }
                     var_item => {
-                        return InvalidPduVariable { var_item }.fail();
+                        return InvalidPduVariableSnafu { var_item }.fail();
                     }
                 }
             }
@@ -216,7 +246,7 @@ where
             Ok(Pdu::AssociationRQ {
                 protocol_version,
                 application_context_name: application_context_name
-                    .context(MissingApplicationContextName)?,
+                    .context(MissingApplicationContextNameSnafu)?,
                 called_ae_title,
                 calling_ae_title,
                 presentation_contexts,
@@ -235,7 +265,7 @@ where
             // Version 1 and shall be identified with bit 0 set. A receiver of this PDU
             // implementing only this version of the DICOM UL protocol shall only test that bit
             // 0 is set.
-            let protocol_version = cursor.read_u16::<BigEndian>().context(ReadPduField {
+            let protocol_version = cursor.read_u16::<BigEndian>().context(ReadPduFieldSnafu {
                 field: "Protocol-version",
             })?;
 
@@ -243,20 +273,48 @@ where
             // tested to this value when received.
             cursor
                 .read_u16::<BigEndian>()
-                .context(ReadReserved { bytes: 2_u32 })?;
+                .context(ReadReservedSnafu { bytes: 2_u32 })?;
 
             // 11-26 - Reserved - This reserved field shall be sent with a value identical to
             // the value received in the same field of the A-ASSOCIATE-RQ PDU, but its value
             // shall not be tested when received.
+            let mut ae_bytes = [0; 16];
+            cursor
+                .read_exact(&mut ae_bytes)
+                .context(ReadPduFieldSnafu {
+                    field: "Called-AE-title",
+                })?;
+            let called_ae_title = codec
+                .decode(&ae_bytes)
+                .context(DecodeTextSnafu {
+                    field: "Called-AE-title",
+                })?
+                .trim()
+                .to_string();
+
             // 27-42 - Reserved - This reserved field shall be sent with a value identical to
             // the value received in the same field of the A-ASSOCIATE-RQ PDU, but its value
             // shall not be tested when received.
+            let mut ae_bytes = [0; 16];
+            cursor
+                .read_exact(&mut ae_bytes)
+                .context(ReadPduFieldSnafu {
+                    field: "Calling-AE-title",
+                })?;
+            let calling_ae_title = codec
+                .decode(&ae_bytes)
+                .context(DecodeTextSnafu {
+                    field: "Calling-AE-title",
+                })?
+                .trim()
+                .to_string();
+
             // 43-74 - Reserved - This reserved field shall be sent with a value identical to
             // the value received in the same field of the A-ASSOCIATE-RQ PDU, but its value
             // shall not be tested when received.
             cursor
-                .seek(SeekFrom::Current(16 + 16 + 32))
-                .context(ReadReserved { bytes: 64_u32 })?;
+                .seek(SeekFrom::Current(32))
+                .context(ReadReservedSnafu { bytes: 32_u32 })?;
 
             // 75-xxx - Variable items - This variable field shall contain the following items:
             // one Application Context Item, one or more Presentation Context Item(s) and one
@@ -274,7 +332,7 @@ where
                         user_variables = val;
                     }
                     var_item => {
-                        return InvalidPduVariable { var_item }.fail();
+                        return InvalidPduVariableSnafu { var_item }.fail();
                     }
                 }
             }
@@ -282,7 +340,9 @@ where
             Ok(Pdu::AssociationAC {
                 protocol_version,
                 application_context_name: application_context_name
-                    .context(MissingApplicationContextName)?,
+                    .context(MissingApplicationContextNameSnafu)?,
+                called_ae_title,
+                calling_ae_title,
                 presentation_contexts,
                 user_variables,
             })
@@ -292,16 +352,20 @@ where
 
             // 7 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
-            cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+            cursor
+                .read_u8()
+                .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
             // 8 - Result - This Result field shall contain an integer value encoded as an unsigned
             // binary number. One of the following values shall be used:
             //   1 - rejected-permanent
             //   2 - rejected-transient
             let result = AssociationRJResult::from(
-                cursor.read_u8().context(ReadPduField { field: "Result" })?,
+                cursor
+                    .read_u8()
+                    .context(ReadPduFieldSnafu { field: "Result" })?,
             )
-            .context(InvalidRejectSourceOrReason)?;
+            .context(InvalidRejectSourceOrReasonSnafu)?;
 
             // 9 - Source - This Source field shall contain an integer value encoded as an unsigned
             // binary number. One of the following values shall be used:   1 - DICOM UL
@@ -325,12 +389,14 @@ where
             //     2 - local-limit-exceeded
             //     3-7 - reserved
             let source = AssociationRJSource::from(
-                cursor.read_u8().context(ReadPduField { field: "Source" })?,
-                cursor.read_u8().context(ReadPduField {
+                cursor
+                    .read_u8()
+                    .context(ReadPduFieldSnafu { field: "Source" })?,
+                cursor.read_u8().context(ReadPduFieldSnafu {
                     field: "Reason/Diag.",
                 })?,
             )
-            .context(InvalidRejectSourceOrReason)?;
+            .context(InvalidRejectSourceOrReasonSnafu)?;
 
             Ok(Pdu::AssociationRJ { result, source })
         }
@@ -347,14 +413,14 @@ where
                 // 1-4 - Item-length - This Item-length shall be the number of bytes from the first
                 // byte of the following field to the last byte of the Presentation-data-value
                 // field. It shall be encoded as an unsigned binary number.
-                let item_length = cursor.read_u32::<BigEndian>().context(ReadPduField {
+                let item_length = cursor.read_u32::<BigEndian>().context(ReadPduFieldSnafu {
                     field: "Item-Length",
                 })?;
 
                 // 5 - Presentation-context-ID - Presentation-context-ID values shall be odd
                 // integers between 1 and 255, encoded as an unsigned binary number. For a complete
                 // description of the use of this field see Section 7.1.1.13.
-                let presentation_context_id = cursor.read_u8().context(ReadPduField {
+                let presentation_context_id = cursor.read_u8().context(ReadPduFieldSnafu {
                     field: "Presentation-context-ID",
                 })?;
 
@@ -373,7 +439,7 @@ where
                 // does not contain the last fragment of a Message Data Set or of a Message Command.
                 let value_type;
                 let is_last;
-                let header = cursor.read_u8().context(ReadPduField {
+                let header = cursor.read_u8().context(ReadPduFieldSnafu {
                     field: "Message Control Header",
                 })?;
 
@@ -389,7 +455,7 @@ where
                 }
 
                 let data =
-                    read_n(&mut cursor, (item_length - 2) as usize).context(ReadPduField {
+                    read_n(&mut cursor, (item_length - 2) as usize).context(ReadPduFieldSnafu {
                         field: "Presentation-data-value",
                     })?;
 
@@ -410,7 +476,7 @@ where
             // tested to this value when received.
             cursor
                 .seek(SeekFrom::Current(4))
-                .context(ReadPduField { field: "Reserved" })?;
+                .context(ReadPduFieldSnafu { field: "Reserved" })?;
 
             Ok(Pdu::ReleaseRQ)
         }
@@ -421,7 +487,7 @@ where
             // tested to this value when received.
             cursor
                 .seek(SeekFrom::Current(4))
-                .context(ReadPduField { field: "Reserved" })?;
+                .context(ReadPduFieldSnafu { field: "Reserved" })?;
 
             Ok(Pdu::ReleaseRP)
         }
@@ -432,13 +498,13 @@ where
             // this value when received.
             cursor
                 .read_u8()
-                .context(ReadPduField { field: "Reserved" })?;
+                .context(ReadPduFieldSnafu { field: "Reserved" })?;
 
             // 8 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
             cursor
                 .read_u8()
-                .context(ReadPduField { field: "Reserved" })?;
+                .context(ReadPduFieldSnafu { field: "Reserved" })?;
 
             // 9 - Source - This Source field shall contain an integer value encoded as an unsigned
             // binary number. One of the following values shall be used:
@@ -455,18 +521,20 @@ where
             // - 5 - unexpected-PDU parameter
             // - 6 - invalid-PDU-parameter value
             let source = AbortRQSource::from(
-                cursor.read_u8().context(ReadPduField { field: "Source" })?,
-                cursor.read_u8().context(ReadPduField {
+                cursor
+                    .read_u8()
+                    .context(ReadPduFieldSnafu { field: "Source" })?,
+                cursor.read_u8().context(ReadPduFieldSnafu {
                     field: "Reason/Diag",
                 })?,
             )
-            .context(InvalidAbortSourceOrReason)?;
+            .context(InvalidAbortSourceOrReasonSnafu)?;
 
             Ok(Pdu::AbortRQ { source })
         }
         _ => {
             let data = read_n(&mut cursor, pdu_length as usize)
-                .context(ReadPduField { field: "Unknown" })?;
+                .context(ReadPduFieldSnafu { field: "Unknown" })?;
             Ok(Pdu::Unknown { pdu_type, data })
         }
     }
@@ -488,17 +556,19 @@ where
     // 1 - Item-type - XXH
     let item_type = reader
         .read_u8()
-        .context(ReadPduField { field: "Item-type" })?;
+        .context(ReadPduFieldSnafu { field: "Item-type" })?;
 
     // 2 - Reserved
-    reader.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+    reader
+        .read_u8()
+        .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
     // 3-4 - Item-length
-    let item_length = reader.read_u16::<BigEndian>().context(ReadPduField {
+    let item_length = reader.read_u16::<BigEndian>().context(ReadPduFieldSnafu {
         field: "Item-length",
     })?;
 
-    let bytes = read_n(reader, item_length as usize).context(ReadPduItem)?;
+    let bytes = read_n(reader, item_length as usize).context(ReadPduItemSnafu)?;
     let mut cursor = Cursor::new(bytes);
 
     match item_type {
@@ -510,9 +580,11 @@ where
             // 7.1.1.2. Application-context-names are structured as UIDs as defined in PS3.5 (see
             // Annex A for an overview of this concept). DICOM Application-context-names are
             // registered in PS3.7.
-            let val = codec.decode(&cursor.into_inner()).context(DecodeText {
-                field: "Application-context-name",
-            })?;
+            let val = codec
+                .decode(&cursor.into_inner())
+                .context(DecodeTextSnafu {
+                    field: "Application-context-name",
+                })?;
             Ok(PduVariableItem::ApplicationContext(val))
         }
         0x20 => {
@@ -524,21 +596,27 @@ where
             // 5 - Presentation-context-ID - Presentation-context-ID values shall be odd integers
             // between 1 and 255, encoded as an unsigned binary number. For a complete description
             // of the use of this field see Section 7.1.1.13.
-            let presentation_context_id = cursor.read_u8().context(ReadPduField {
+            let presentation_context_id = cursor.read_u8().context(ReadPduFieldSnafu {
                 field: "Presentation-context-ID",
             })?;
 
             // 6 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
-            cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+            cursor
+                .read_u8()
+                .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
             // 7 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
-            cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+            cursor
+                .read_u8()
+                .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
             // 8 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
-            cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+            cursor
+                .read_u8()
+                .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
             // 9-xxx - Abstract/Transfer Syntax Sub-Items - This variable field shall contain the
             // following sub-items: one Abstract Syntax and one or more Transfer Syntax(es). For a
@@ -548,14 +626,16 @@ where
                 // 1 - Item-type - XXH
                 let item_type = cursor
                     .read_u8()
-                    .context(ReadPduField { field: "Item-type" })?;
+                    .context(ReadPduFieldSnafu { field: "Item-type" })?;
 
                 // 2 - Reserved - This reserved field shall be sent with a value 00H but not tested
                 // to this value when received.
-                cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+                cursor
+                    .read_u8()
+                    .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
                 // 3-4 - Item-length
-                let item_length = cursor.read_u16::<BigEndian>().context(ReadPduField {
+                let item_length = cursor.read_u16::<BigEndian>().context(ReadPduFieldSnafu {
                     field: "Item-length",
                 })?;
 
@@ -573,11 +653,11 @@ where
                         abstract_syntax = Some(
                             codec
                                 .decode(&read_n(&mut cursor, item_length as usize).context(
-                                    ReadPduField {
+                                    ReadPduFieldSnafu {
                                         field: "Abstract-syntax-name",
                                     },
                                 )?)
-                                .context(DecodeText {
+                                .context(DecodeTextSnafu {
                                     field: "Abstract-syntax-name",
                                 })?
                                 .trim()
@@ -597,11 +677,11 @@ where
                         transfer_syntaxes.push(
                             codec
                                 .decode(&read_n(&mut cursor, item_length as usize).context(
-                                    ReadPduField {
+                                    ReadPduFieldSnafu {
                                         field: "Transfer-syntax-name",
                                     },
                                 )?)
-                                .context(DecodeText {
+                                .context(DecodeTextSnafu {
                                     field: "Transfer-syntax-name",
                                 })?
                                 .trim()
@@ -609,7 +689,7 @@ where
                         );
                     }
                     _ => {
-                        return UnknownPresentationContextSubItem.fail();
+                        return UnknownPresentationContextSubItemSnafu.fail();
                     }
                 }
             }
@@ -617,7 +697,7 @@ where
             Ok(PduVariableItem::PresentationContextProposed(
                 PresentationContextProposed {
                     id: presentation_context_id,
-                    abstract_syntax: abstract_syntax.context(MissingAbstractSyntax)?,
+                    abstract_syntax: abstract_syntax.context(MissingAbstractSyntaxSnafu)?,
                     transfer_syntaxes,
                 },
             ))
@@ -630,13 +710,15 @@ where
             // 5 - Presentation-context-ID - Presentation-context-ID values shall be odd integers
             // between 1 and 255, encoded as an unsigned binary number. For a complete description
             // of the use of this field see Section 7.1.1.13.
-            let presentation_context_id = cursor.read_u8().context(ReadPduField {
+            let presentation_context_id = cursor.read_u8().context(ReadPduFieldSnafu {
                 field: "Presentation-context-ID",
             })?;
 
             // 6 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
-            cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+            cursor
+                .read_u8()
+                .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
             // 7 - Result/Reason - This Result/Reason field shall contain an integer value encoded
             // as an unsigned binary number. One of the following values shall be used:
@@ -645,15 +727,18 @@ where
             //   2 - no-reason (provider rejection)
             //   3 - abstract-syntax-not-supported (provider rejection)
             //   4 - transfer-syntaxes-not-supported (provider rejection)
-            let reason =
-                PresentationContextResultReason::from(cursor.read_u8().context(ReadPduField {
+            let reason = PresentationContextResultReason::from(cursor.read_u8().context(
+                ReadPduFieldSnafu {
                     field: "Result/Reason",
-                })?)
-                .context(InvalidPresentationContextResultReason)?;
+                },
+            )?)
+            .context(InvalidPresentationContextResultReasonSnafu)?;
 
             // 8 - Reserved - This reserved field shall be sent with a value 00H but not tested to
             // this value when received.
-            cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+            cursor
+                .read_u8()
+                .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
             // 9-xxx - Transfer syntax sub-item - This variable field shall contain one Transfer
             // Syntax Sub-Item. When the Result/Reason field has a value other than acceptance (0),
@@ -664,14 +749,16 @@ where
                 // 1 - Item-type - XXH
                 let item_type = cursor
                     .read_u8()
-                    .context(ReadPduField { field: "Item-type" })?;
+                    .context(ReadPduFieldSnafu { field: "Item-type" })?;
 
                 // 2 - Reserved - This reserved field shall be sent with a value 00H but not tested
                 // to this value when received.
-                cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+                cursor
+                    .read_u8()
+                    .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
                 // 3-4 - Item-length
-                let item_length = cursor.read_u16::<BigEndian>().context(ReadPduField {
+                let item_length = cursor.read_u16::<BigEndian>().context(ReadPduFieldSnafu {
                     field: "Item-length",
                 })?;
 
@@ -689,19 +776,19 @@ where
                         match transfer_syntax {
                             Some(_) => {
                                 // Multiple transfer syntax values cannot be proposed.
-                                return MultipleTransferSyntaxesAccepted.fail();
+                                return MultipleTransferSyntaxesAcceptedSnafu.fail();
                             }
                             None => {
                                 transfer_syntax = Some(
                                     codec
                                         .decode(
                                             &read_n(&mut cursor, item_length as usize).context(
-                                                ReadPduField {
+                                                ReadPduFieldSnafu {
                                                     field: "Transfer-syntax-name",
                                                 },
                                             )?,
                                         )
-                                        .context(DecodeText {
+                                        .context(DecodeTextSnafu {
                                             field: "Transfer-syntax-name",
                                         })?
                                         .trim()
@@ -711,7 +798,7 @@ where
                         }
                     }
                     _ => {
-                        return InvalidTransferSyntaxSubItem.fail();
+                        return InvalidTransferSyntaxSubItemSnafu.fail();
                     }
                 }
             }
@@ -720,7 +807,7 @@ where
                 PresentationContextResult {
                     id: presentation_context_id,
                     reason,
-                    transfer_syntax: transfer_syntax.context(MissingTransferSyntax)?,
+                    transfer_syntax: transfer_syntax.context(MissingTransferSyntaxSnafu)?,
                 },
             ))
         }
@@ -736,13 +823,15 @@ where
                 // 1 - Item-type - XXH
                 let item_type = cursor
                     .read_u8()
-                    .context(ReadPduField { field: "Item-type" })?;
+                    .context(ReadPduFieldSnafu { field: "Item-type" })?;
 
                 // 2 - Reserved
-                cursor.read_u8().context(ReadReserved { bytes: 1_u32 })?;
+                cursor
+                    .read_u8()
+                    .context(ReadReservedSnafu { bytes: 1_u32 })?;
 
                 // 3-4 - Item-length
-                let item_length = cursor.read_u16::<BigEndian>().context(ReadPduField {
+                let item_length = cursor.read_u16::<BigEndian>().context(ReadPduFieldSnafu {
                     field: "Item-length",
                 })?;
 
@@ -760,7 +849,7 @@ where
                         // received by the association-requestor. Otherwise, it shall be a protocol
                         // error.
                         user_variables.push(UserVariableItem::MaxLength(
-                            cursor.read_u32::<BigEndian>().context(ReadPduField {
+                            cursor.read_u32::<BigEndian>().context(ReadPduFieldSnafu {
                                 field: "Maximum-length-received",
                             })?,
                         ));
@@ -774,11 +863,11 @@ where
                         // UID as defined in PS3.5.
                         let implementation_class_uid = codec
                             .decode(&read_n(&mut cursor, item_length as usize).context(
-                                ReadPduField {
+                                ReadPduFieldSnafu {
                                     field: "Implementation-class-uid",
                                 },
                             )?)
-                            .context(DecodeText {
+                            .context(DecodeTextSnafu {
                                 field: "Implementation-class-uid",
                             })?
                             .trim()
@@ -796,11 +885,11 @@ where
                         // (basic G0 set) characters.
                         let implementation_version_name = codec
                             .decode(&read_n(&mut cursor, item_length as usize).context(
-                                ReadPduField {
+                                ReadPduFieldSnafu {
                                     field: "Implementation-version-name",
                                 },
                             )?)
-                            .context(DecodeText {
+                            .context(DecodeTextSnafu {
                                 field: "Implementation-version-name",
                             })?
                             .trim()
@@ -813,7 +902,7 @@ where
                         user_variables.push(UserVariableItem::Unknown(
                             item_type,
                             read_n(&mut cursor, item_length as usize)
-                                .context(ReadPduField { field: "Unknown" })?,
+                                .context(ReadPduFieldSnafu { field: "Unknown" })?,
                         ));
                     }
                 }
